# Copyright (C) 2001-2003 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

# $Id: opcode.py,v 1.2 2003/05/29 00:31:40 halley Exp $

"""DNS Opcodes."""

QUERY = 0
IQUERY = 1
STATUS = 2
NOTIFY = 4
UPDATE = 5

by_text = {
    'QUERY' : QUERY,
    'IQUERY' : IQUERY,
    'STATUS' : STATUS,
    'NOTIFY' : NOTIFY,
    'UPDATE' : UPDATE
}

by_value = {
    QUERY : 'QUERY',
    IQUERY : 'IQUERY',
    STATUS : 'STATUS',
    NOTIFY : 'NOTIFY',
    UPDATE : 'UPDATE'
}

class UnknownOpcode(Exception):
    """Raised if an opcode is unknown."""
    pass

def from_text(text):
    """Convert text into an opcode."""
    value = by_text.get(text.upper())
    if value is None:
        raise UnknownOpcode
    return value

def from_flags(flags):
    """Extract an opcode from DNS message flags."""
    return (flags & 0x7800) >> 11

def to_flags(value):
    """Convert an opcode to a value suitable for ORing into DNS message
    flags."""
    return (value << 11) & 0x7800
    
def to_text(value):
    """Convert an opcode to text."""
    text = by_value.get(value)
    if text is None:
        raise UnknownOpcode
    return text

def is_update(flags):
    """True if the opcode in flags is UPDATE."""
    if (from_flags(flags) == UPDATE):
        return True
    return False
